"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgV4SimpleSubscriptionsPlugin = void 0;
const json_1 = require("@dataplan/json");
const grafast_1 = require("grafast");
const graphile_utils_1 = require("graphile-utils");
exports.PgV4SimpleSubscriptionsPlugin = (0, graphile_utils_1.makeExtendSchemaPlugin)((build) => {
    const nodeIdHandlerByTypeName = build.getNodeIdHandlerByTypeName?.();
    return {
        typeDefs: [
            (0, graphile_utils_1.gql) `
        extend type Subscription {
          listen(topic: String!): ListenPayload
        }

        type ListenPayload {
          event: String
        }
      `,
            ... // Only add the relatedNode if supported
            (nodeIdHandlerByTypeName
                ? [
                    (0, graphile_utils_1.gql) `
              extend type ListenPayload {
                relatedNode: Node
                relatedNodeId: ID
              }
            `,
                ]
                : []),
        ],
        plans: {
            Subscription: {
                listen: {
                    subscribePlan: (0, graphile_utils_1.EXPORTABLE)((context, jsonParse, lambda, listen) => function subscribePlan(_$root, { $topic }) {
                        const $pgSubscriber = context().get("pgSubscriber");
                        const $derivedTopic = lambda($topic, (topic) => `postgraphile:${topic}`);
                        return listen($pgSubscriber, $derivedTopic, jsonParse);
                    }, [grafast_1.context, json_1.jsonParse, grafast_1.lambda, grafast_1.listen]),
                    plan: (0, graphile_utils_1.EXPORTABLE)(() => function plan($event) {
                        return $event;
                    }, []),
                },
            },
            ListenPayload: {
                event($event) {
                    return $event.get("event");
                },
                ...(nodeIdHandlerByTypeName
                    ? {
                        relatedNodeId($event) {
                            return nodeIdFromEvent($event);
                        },
                        relatedNode($event) {
                            const $nodeId = nodeIdFromEvent($event);
                            return (0, grafast_1.node)(nodeIdHandlerByTypeName, $nodeId);
                        },
                    }
                    : null),
            },
        },
    };
}, "PgV4SimpleSubscriptionsPlugin");
// WARNING: this function assumes that you're using the `base64JSON` NodeID codec, which was the case for PostGraphile V4. If you are not doing so, YMMV.
function nodeIdFromEvent($event) {
    const $nodeObj = $event.get("__node__");
    const $nodeId = (0, grafast_1.lambda)($nodeObj, nodeObjToNodeId);
    return $nodeId;
}
// base64JSON codec copy
function nodeObjToNodeId(obj) {
    if (!obj)
        return null;
    return Buffer.from(JSON.stringify(obj), "utf8").toString("base64");
}
//# sourceMappingURL=PgV4SimpleSubscriptionsPlugin.js.map